#ifndef _GUASH_CONFIG_H_
#define _GUASH_CONFIG_H_
/* 
 * CUSTOMIZABLE CONSTANTS
 */

/* set font height manually ONLY IF you use gtk+1.0.X 
   (gtk+-1.1.0 or upper has gdk_string_height. guash uses it.). */
//#define THUMBNAIL_FONT_SIZE 16		/* [pixel] */

/*
 * *****************************************************************
 */

/* If you didn't set LANG(?) to latin-1(?), comment will be broken...
   TO avoid, translate "File written by Photoshop(c) X.X" to 
   "File written by Photoshop  X.X" in show_comment_callback.
   Modify freely, I don't understand true condition. */
#ifndef HAVE_LC_MESSAGES
//#define TRANSLATE_PHOTOSHOP_TM	1
#endif

#define LOCALEDIR g_strconcat (gimp_data_directory (), \
			       G_DIR_SEPARATOR_S, \
			       "locale", \
			       NULL)

#define NEW_DIRECTORY_MODE	00777
#define NEW_FILE_MODE		00644

/* background color of thumbnail panel */
#define BASE_COLOR		{ 255, 255, 255 }
/* foreground color (the color to draw string in the thumbnail panel) */
#define DRAW_COLOR		{ 0, 0, 0 }
/* the color to draw the names of symbolic links */
#define SLINK_COLOR		{ 46, 139, 87 }
/* the color to show selection frame */
#define FRAME_COLOR		{ 255, 0, 0 }
#define USE_UPDATE_BUTTON	0 /* 1 */

/*
 * **********************************************************************
 * the followings are optional.
 * **********************************************************************
 */

/* You can set inihibit suffixes by guash-inhibit-suffix in gimprc */
#define GUASH_INHIBIT_SUFFIX_TABLE_DEFAULT \
	"core:.mpeg:.mpg:.tar:.tar.gz:.tgz:.tar.bz2:.h"

/* You can set the default command by guash-mapping-command in gimprc 
 * {} in the string is replaced by the selected file name in execution time.
 */ 
#define MAPPING_COMMAND_DEFAULT \
	"xv -root -maxpect -quit {}"

/* You can set the history size by guash-directory-history-length in gimprc
 * If your machine has small (physical) memory, try smaller value (use top). */
#define DIRECTORY_CACHE_TABLE_MAX_SIZE	6

/* You can set the polling interval by guash-directory-monitor-interval in
 * gimprc. 10 seconds seems too long to me but would be modest.
 * You need not change the value, even if you don't want to use
 * directory-monitor. Whether use it or not is controlled by the 6th letter
 * in guash-option-string. */
#define DIRECTORY_MONITOR_INTERVAL_DEFAULT 10 /* [second] */

/* You can set the default size by guash-ncol and guash-nrow in gimprc */
#define NCOL_OF_THUMBNAIL_PANEL_DEFAULT	5
#define NROW_OF_THUMBNAIL_PANEL_DEFAULT	3

/* 
 * ARCHITECTURE-DEPENDED CONSTANTS
 */
#define GCHAR	gchar

/*
 * ABSORBERS OF VERSION DIFFERENCES OF GLIB/GTK+/GIMP
 */

/* GIMP_*_VERSION was introduced in gimp-1.0.1
 * libgimp/gimpintl.h was introduced in gimp-1.1.0
 */
#if defined(GIMP_MAJOR_VERSION) && ((1 < GIMP_MAJOR_VERSION) || (0 < GIMP_MINOR_VERSION))
# include <libgimp/gimpintl.h>
#else
# define _(String) (String)
# define N_(String) (String)
# define textdomain(String) (String)
# define gettext(String) (String)
# define dgettext(Domain,String) (String)
# define dcgettext(Domain,String,Type) (String)
# define bindtextdomain(Domain,Directory) (Domain)
#endif

/* glib 1.0 wasn't ported to Win*, so I use NATIVE_WIN32 for version checker */
#ifndef NATIVE_WIN32
#define G_DIR_SEPARATOR_S "/"
#define g_strncasecmp strncmp
#endif

/* 
 * MISC SWITCHES
 */
#define	DRAW_STRING_BY_CHARACTER	1

/* *
#define ENABLE_NLS 1
* */
/* *
#define DEBUG 1
* */
/* *
#define DEATH_ATTACK	1
* */
/* *
#define CHECK_PERFORMANCE 1
* */

/* BROKEN_SCRIPT_FU means:
 * result of script invoked from menu != result of script called from a plug-in
 * All gimps since 1.0.1 (if my memory is right) are BROKEN
 */
#define BROKEN_SCRIPT_FU	1

#ifdef DEBUG
#define DEBUG_SUPPRESS_LEVEL	9
#undef DIRECTORY_MONITOR_INTERVAL_DEFAULT
#define DIRECTORY_MONITOR_INTERVAL_DEFAULT 6
#undef DIRECTORY_CACHE_TABLE_MAX_SIZE
#define DIRECTORY_CACHE_TABLE_MAX_SIZE	2
#endif

#ifndef GIMP_MAJOR_VERSION
#undef ENABLE_NLS
#endif

#ifdef ENABLE_NLS
#define TEXT_DOMAIN	"guash"
#endif

#ifdef GTK_HAVE_FEATURES_1_1_0
#undef THUMBNAIL_FONT_SIZE
#endif

#ifdef GTK_HAVE_FEATURES_1_1_2
#define NEW_DND_API
#endif

#ifndef G_DIR_SEPARATOR
#define G_DIR_SEPARATOR '/'
#endif

#ifndef CONFIG_H
#define CONFIG_H
#endif

/*
 * Local variables:
 * compile-command: "gimptool --install guash.c"
 * End:
 */
#endif /* _GUASH_CONFIG_H_ */
/* config.h ends here */
